@@ -40,7 +40,7 @@ JOB_WAIT, JOB_ERROR, JOB_INPUT_ERROR, JOB_INPUT_DELETED, JOB_READY, JOB_DELETED,
 DEFAULT_JOB_PUT_FAILURE_MESSAGE = 'Unable to run job due to a misconfiguration of the Galaxy job running system.  Please contact a site administrator.'
 
 
-class JobHandler(object):
+class JobHandler:
     """
     Handle the preparation, running, tracking, and finishing of jobs
     """
@@ -62,6 +62,72 @@ class JobHandler(object):
         self.job_stop_queue.shutdown()
 
 
+class ItemGrabber:
+
+    def __init__(self, app, grab_type='Job', handler_assignment_method=None, max_grab=None, self_handler_tags=None, handler_tags=None):
+        self.app = app
+        self.sa_session = app.model.context
+        self.grab_this = getattr(model, grab_type)
+        self.grab_type = grab_type
+        self._grab_conn_opts = {'autocommit': False}
+        subq = select([self.grab_this.id]) \
+            .where(and_(
+                self.grab_this.table.c.handler.in_(self_handler_tags),
+                self.grab_this.table.c.state == self.grab_this.states.NEW)) \
+            .order_by(self.grab_this.table.c.id)
+        if max_grab:
+            subq = subq.limit(max_grab)
+        if handler_assignment_method == HANDLER_ASSIGNMENT_METHODS.DB_SKIP_LOCKED:
+            subq = subq.with_for_update(skip_locked=True)
+        self._grab_query = self.grab_this.table.update() \
+            .returning(self.grab_this.table.c.id) \
+            .where(self.grab_this.table.c.id.in_(subq)) \
+            .values(handler=self.app.config.server_name)
+        if handler_assignment_method == HANDLER_ASSIGNMENT_METHODS.DB_TRANSACTION_ISOLATION:
+            self._grab_conn_opts['isolation_level'] = 'SERIALIZABLE'
+        log.info(
+            "Handler job grabber initialized with '%s' assignment method for handler '%s', tag(s): %s", handler_assignment_method,
+            self.app.config.server_name, ', '.join(str(x) for x in handler_tags)
+        )
+
+    @staticmethod
+    def get_grabbable_handler_assignment_method(handler_assignment_methods):
+        grabbable_methods = {
+            HANDLER_ASSIGNMENT_METHODS.DB_TRANSACTION_ISOLATION,
+            HANDLER_ASSIGNMENT_METHODS.DB_SKIP_LOCKED,
+        }
+        if handler_assignment_methods:
+            try:
+                return [m for m in handler_assignment_methods if m in grabbable_methods][0]
+            except IndexError:
+                return
+
+    def grab_unhandled_items(self):
+        """
+        Attempts to assign unassigned jobs or invocaions to itself using DB serialization methods, if enabled. This
+        simply sets `Job.handler` or `WorkflowInvocation.handler` to the current server name, which causes the job to be picked up by
+        the appropriate handler.
+        """
+        # an excellent discussion on PostgreSQL concurrency safety:
+        # https://blog.2ndquadrant.com/what-is-select-skip-locked-for-in-postgresql-9-5/
+        self.sa_session.expunge_all()
+        conn = self.sa_session.connection(execution_options=self._grab_conn_opts)
+        with conn.begin() as trans:
+            try:
+                rows = conn.execute(self._grab_query).fetchall()
+                if rows:
+                    log.debug('Grabbed %s(s): %s', self.grab_type, ', '.join(str(row[0]) for row in rows))
+                    trans.commit()
+                else:
+                    trans.rollback()
+            except OperationalError as e:
+                # If this is a serialization failure on PostgreSQL, then e.orig is a psycopg2 TransactionRollbackError
+                # and should have attribute `code`. Other engines should just report the message and move on.
+                if int(getattr(e.orig, 'pgcode', -1)) != 40001:
+                    log.debug('Grabbing %s failed (serialization failures are ok): %s', self.grab_type, unicodify(e))
+                trans.rollback()
+
+
 class JobHandlerQueue(Monitors):
     """
     Job Handler's Internal Queue, this is what actually implements waiting for
@@ -91,38 +157,17 @@ class JobHandlerQueue(Monitors):
         self.job_wrappers = {}
         name = "JobHandlerQueue.monitor_thread"
         self._init_monitor_thread(name, target=self.__monitor, config=app.config)
-        self.__grab_query = None
-        self.__grab_conn_opts = {'autocommit': False}
-        self.__initialize_job_grabbing()
-
-    def __initialize_job_grabbing(self):
-        grabbable_methods = {
-            HANDLER_ASSIGNMENT_METHODS.DB_TRANSACTION_ISOLATION,
-            HANDLER_ASSIGNMENT_METHODS.DB_SKIP_LOCKED,
-        }
-        try:
-            method = [m for m in self.app.job_config.handler_assignment_methods if m in grabbable_methods][0]
-        except IndexError:
-            return
-        subq = select([model.Job.id]) \
-            .where(and_(
-                model.Job.table.c.handler.in_(self.app.job_config.self_handler_tags),
-                model.Job.table.c.state == model.Job.states.NEW)) \
-            .order_by(model.Job.table.c.id)
-        if self.app.job_config.handler_max_grab:
-            subq = subq.limit(self.app.job_config.handler_max_grab)
-        if method == HANDLER_ASSIGNMENT_METHODS.DB_SKIP_LOCKED:
-            subq = subq.with_for_update(skip_locked=True)
-        self.__grab_query = model.Job.table.update() \
-            .returning(model.Job.table.c.id) \
-            .where(model.Job.table.c.id.in_(subq)) \
-            .values(handler=self.app.config.server_name)
-        if method == HANDLER_ASSIGNMENT_METHODS.DB_TRANSACTION_ISOLATION:
-            self.__grab_conn_opts['isolation_level'] = 'SERIALIZABLE'
-        log.info(
-            "Handler job grabber initialized with '%s' assignment method for handler '%s', tag(s): %s", method,
-            self.app.config.server_name, ', '.join(str(x) for x in self.app.job_config.handler_tags)
-        )
+        self.job_grabber = None
+        handler_assignment_method = ItemGrabber.get_grabbable_handler_assignment_method(self.app.job_config.handler_assignment_methods)
+        if handler_assignment_method:
+            self.job_grabber = ItemGrabber(
+                app=app,
+                grab_type='Job',
+                handler_assignment_method=handler_assignment_method,
+                max_grab=self.app.job_config.handler_max_grab,
+                self_handler_tags=self.app.job_config.self_handler_tags,
+                handler_tags=self.app.job_config.handler_tags,
+            )
 
     def start(self):
         """
@@ -173,7 +218,7 @@ class JobHandlerQueue(Monitors):
 
         for job in jobs_at_startup:
             if not self.app.toolbox.has_tool(job.tool_id, job.tool_version, exact=True):
-                log.warning("(%s) Tool '%s' removed from tool config, unable to recover job" % (job.id, job.tool_id))
+                log.warning("({}) Tool '{}' removed from tool config, unable to recover job".format(job.id, job.tool_id))
                 self.job_wrapper(job).fail('This tool was disabled before the job completed.  Please contact your Galaxy administrator.')
             elif job.job_runner_name is not None and job.job_runner_external_id is None:
                 # This could happen during certain revisions of Galaxy where a runner URL was persisted before the job was dispatched to a runner.
@@ -194,7 +239,7 @@ class JobHandlerQueue(Monitors):
                 log.info('(%s) Converted job from a URL to a destination and recovered' % (job.id))
             elif job.job_runner_name is None:
                 # Never (fully) dispatched
-                log.debug("(%s) No job runner assigned and job still in '%s' state, adding to the job handler queue" % (job.id, job.state))
+                log.debug("({}) No job runner assigned and job still in '{}' state, adding to the job handler queue".format(job.id, job.state))
                 if self.track_jobs_in_database:
                     job.set_state(model.Job.states.NEW)
                 else:
@@ -248,36 +293,11 @@ class JobHandlerQueue(Monitors):
             'internal.galaxy.jobs.handlers.monitor_step',
             'Job handler monitor step complete.'
         )
-        if self.__grab_query is not None:
-            self.__grab_unhandled_jobs()
+        if self.job_grabber is not None:
+            self.job_grabber.grab_unhandled_items()
         self.__handle_waiting_jobs()
         log.trace(monitor_step_timer.to_str())
 
-    def __grab_unhandled_jobs(self):
-        """
-        Attempts to "grab" jobs (assign unassigned jobs to itself) using DB serialization methods, if enabled. This
-        simply sets `Job.handler` to the current server name, which causes the job to be picked up by
-        `__handle_waiting_jobs()`.
-        """
-        # an excellent discussion on PostgreSQL concurrency safety:
-        # https://blog.2ndquadrant.com/what-is-select-skip-locked-for-in-postgresql-9-5/
-        self.sa_session.expunge_all()
-        conn = self.sa_session.connection(execution_options=self.__grab_conn_opts)
-        with conn.begin() as trans:
-            try:
-                rows = conn.execute(self.__grab_query).fetchall()
-                if rows:
-                    log.debug('Grabbed job(s): %s', ', '.join(str(row[0]) for row in rows))
-                    trans.commit()
-                else:
-                    trans.rollback()
-            except OperationalError as e:
-                # If this is a serialization failure on PostgreSQL, then e.orig is a psycopg2 TransactionRollbackError
-                # and should have attribute `code`. Other engines should just report the message and move on.
-                if int(getattr(e.orig, 'pgcode', -1)) != 40001:
-                    log.debug('Grabbing job failed (serialization failures are ok): %s', unicodify(e))
-                trans.rollback()
-
     def __handle_waiting_jobs(self):
         """
         Gets any new jobs (either from the database or from its own queue), then iterates over all new and waiting jobs
@@ -467,7 +487,7 @@ class JobHandlerQueue(Monitors):
             elif dataset_state == model.Dataset.states.ERROR:
                 jobs_to_pause[job_id].append("Input dataset '%s' is in error state" % hda_name)
             elif dataset_state != model.Dataset.states.OK:
-                jobs_to_ignore[job_id].append("Input dataset '%s' is in %s state" % (hda_name, dataset_state))
+                jobs_to_ignore[job_id].append("Input dataset '{}' is in {} state".format(hda_name, dataset_state))
         for job_id in sorted(jobs_to_pause):
             pause_message = ", ".join(jobs_to_pause[job_id])
             pause_message = "%s. To resume this job fix the input dataset(s)." % pause_message
@@ -538,7 +558,7 @@ class JobHandlerQueue(Monitors):
             # Cause the job_destination to be set and cached by the mapper
             job_destination = job_wrapper.job_destination
         except AssertionError as e:
-            log.warning("(%s) Tool '%s' removed from tool config, unable to run job" % (job.id, job.tool_id))
+            log.warning("({}) Tool '{}' removed from tool config, unable to run job".format(job.id, job.tool_id))
             job_wrapper.fail(e)
             return JOB_ERROR, job_destination
         except JobNotReadyException as e:
@@ -612,7 +632,7 @@ class JobHandlerQueue(Monitors):
                 continue
             # don't run jobs for which the input dataset was deleted
             if idata.deleted:
-                self.job_wrappers.pop(job.id, self.job_wrapper(job)).fail("input data %s (file: %s) was deleted before the job started" % (idata.hid, idata.file_name))
+                self.job_wrappers.pop(job.id, self.job_wrapper(job)).fail("input data {} (file: {}) was deleted before the job started".format(idata.hid, idata.file_name))
                 return JOB_INPUT_DELETED
             # an error in the input data causes us to bail immediately
             elif idata.state == idata.states.ERROR:
@@ -945,7 +965,7 @@ class JobHandlerStopQueue(Monitors):
             log.info("job handler stop queue stopped")
 
 
-class DefaultJobDispatcher(object):
+class DefaultJobDispatcher:
 
     def __init__(self, app):
         self.app = app
@@ -981,12 +1001,12 @@ class DefaultJobDispatcher(object):
         try:
             if isinstance(job_wrapper, TaskWrapper):
                 # DBTODO Refactor
-                log.debug("(%s) Dispatching task %s to %s runner" % (job_wrapper.job_id, job_wrapper.task_id, runner_name))
+                log.debug("({}) Dispatching task {} to {} runner".format(job_wrapper.job_id, job_wrapper.task_id, runner_name))
             else:
-                log.debug("(%s) Dispatching to %s runner" % (job_wrapper.job_id, runner_name))
+                log.debug("({}) Dispatching to {} runner".format(job_wrapper.job_id, runner_name))
             self.job_runners[runner_name].put(job_wrapper)
         except KeyError:
-            log.error('put(): (%s) Invalid job runner: %s' % (job_wrapper.job_id, runner_name))
+            log.error('put(): ({}) Invalid job runner: {}'.format(job_wrapper.job_id, runner_name))
             job_wrapper.fail(DEFAULT_JOB_PUT_FAILURE_MESSAGE)
 
     def stop(self, job, job_wrapper):
@@ -1004,11 +1024,11 @@ class DefaultJobDispatcher(object):
         job_runner_name = job.get_job_runner_name()
         if job_runner_name is not None:
             runner_name = job_runner_name.split(":", 1)[0]
-            log.debug("Stopping job %s in %s runner" % (job_wrapper.get_id_tag(), runner_name))
+            log.debug("Stopping job {} in {} runner".format(job_wrapper.get_id_tag(), runner_name))
             try:
                 self.job_runners[runner_name].stop_job(job_wrapper)
             except KeyError:
-                log.error('stop(): (%s) Invalid job runner: %s' % (job_wrapper.get_id_tag(), runner_name))
+                log.error('stop(): ({}) Invalid job runner: {}'.format(job_wrapper.get_id_tag(), runner_name))
                 # Job and output dataset states have already been updated, so nothing is done here.
 
     def recover(self, job, job_wrapper):
@@ -1017,7 +1037,7 @@ class DefaultJobDispatcher(object):
         try:
             self.job_runners[runner_name].recover(job, job_wrapper)
         except KeyError:
-            log.error('recover(): (%s) Invalid job runner: %s' % (job_wrapper.job_id, runner_name))
+            log.error('recover(): ({}) Invalid job runner: {}'.format(job_wrapper.job_id, runner_name))
             job_wrapper.fail(DEFAULT_JOB_PUT_FAILURE_MESSAGE)
 
     def shutdown(self):
