@@ -1,5 +1,3 @@
-from __future__ import absolute_import
-
 import logging
 import os
 import time
@@ -103,7 +101,7 @@ class PBSJobRunner(AsynchronousJobRunner):
         self.default_pbs_server     # this is a method with a property decorator, so this causes the default server to be set
 
         # Proceed with general initialization
-        super(PBSJobRunner, self).__init__(app, nworkers)
+        super().__init__(app, nworkers)
         self._init_monitor_thread()
         self._init_worker_threads()
 
@@ -132,7 +130,7 @@ class PBSJobRunner(AsynchronousJobRunner):
         pbs_destination = '@%s' % server
         pbs_queue = url_split[3] or None
         if pbs_queue is not None:
-            pbs_destination = '%s%s' % (pbs_queue, pbs_destination)
+            pbs_destination = '{}{}'.format(pbs_queue, pbs_destination)
 
         params = dict(destination=pbs_destination)
 
@@ -149,7 +147,7 @@ class PBSJobRunner(AsynchronousJobRunner):
             param, value = opt.split(None, 1)
             params[param] = value
 
-        log.debug("Converted URL '%s' to destination runner=pbs, params=%s" % (url, params))
+        log.debug("Converted URL '{}' to destination runner=pbs, params={}".format(url, params))
 
         # Create a dynamic JobDestination
         return JobDestination(runner='pbs', params=params)
@@ -182,13 +180,13 @@ class PBSJobRunner(AsynchronousJobRunner):
         for arg, value in args.items():
             if arg == 'l':
                 resource_attrs = value.split(',')
-                for j, (res, val) in enumerate([a.split('=', 1) for a in resource_attrs]):
+                for res, val in [a.split('=', 1) for a in resource_attrs]:
                     rval.append(dict(name=pbs.ATTR_l, value=val, resource=res))
             else:
                 try:
                     rval.append(dict(name=getattr(pbs, 'ATTR_' + arg), value=value))
                 except AttributeError as e:
-                    raise Exception("Invalid parameter '%s': %s" % (arg, e))
+                    raise Exception("Invalid parameter '{}': {}".format(arg, e))
         return rval
 
     def __get_pbs_server(self, job_destination_params):
@@ -226,19 +224,19 @@ class PBSJobRunner(AsynchronousJobRunner):
         pbs_options = self.parse_destination_params(job_destination.params)
 
         # Explicitly set the determined PBS destination in the persisted job destination for recovery
-        job_destination.params['destination'] = '%s@%s' % (pbs_queue_name or '', pbs_server_name)
+        job_destination.params['destination'] = '{}@{}'.format(pbs_queue_name or '', pbs_server_name)
 
         c = pbs.pbs_connect(util.smart_str(pbs_server_name))
         if c <= 0:
             errno, text = pbs.error()
             job_wrapper.fail("Unable to queue job for execution.  Resubmitting the job may succeed.")
-            log.error("Connection to PBS server for submit failed: %s: %s" % (errno, text))
+            log.error("Connection to PBS server for submit failed: {}: {}".format(errno, text))
             return
 
         # define job attributes
-        ofile = "%s/%s.o" % (self.app.config.cluster_files_directory, job_wrapper.job_id)
-        efile = "%s/%s.e" % (self.app.config.cluster_files_directory, job_wrapper.job_id)
-        ecfile = "%s/%s.ec" % (self.app.config.cluster_files_directory, job_wrapper.job_id)
+        ofile = "{}/{}.o".format(self.app.config.cluster_files_directory, job_wrapper.job_id)
+        efile = "{}/{}.e".format(self.app.config.cluster_files_directory, job_wrapper.job_id)
+        ecfile = "{}/{}.ec".format(self.app.config.cluster_files_directory, job_wrapper.job_id)
 
         output_fnames = job_wrapper.get_output_fnames()
 
@@ -264,7 +262,7 @@ class PBSJobRunner(AsynchronousJobRunner):
             ]
 
         # define PBS job options
-        attrs.append(dict(name=pbs.ATTR_N, value=str("%s_%s_%s" % (job_wrapper.job_id, job_wrapper.tool.id, job_wrapper.user))))
+        attrs.append(dict(name=pbs.ATTR_N, value=str("{}_{}_{}".format(job_wrapper.job_id, job_wrapper.tool.id, job_wrapper.user))))
         job_attrs = pbs.new_attropl(len(attrs) + len(pbs_options))
         for i, attr in enumerate(attrs + pbs_options):
             job_attrs[i].name = attr['name']
@@ -289,7 +287,7 @@ class PBSJobRunner(AsynchronousJobRunner):
 
         env_setup_commands = [stage_commands]
         script = self.get_job_file(job_wrapper, exit_code_path=ecfile, env_setup_commands=env_setup_commands, shell=job_wrapper.shell)
-        job_file = "%s/%s.sh" % (self.app.config.cluster_files_directory, job_wrapper.job_id)
+        job_file = "{}/{}.sh".format(self.app.config.cluster_files_directory, job_wrapper.job_id)
         self.write_executable_script(job_file, script)
         # job was deleted while we were preparing it
         if job_wrapper.get_state() == model.Job.states.DELETED:
@@ -304,7 +302,7 @@ class PBSJobRunner(AsynchronousJobRunner):
         # The job tag includes the job and the task identifier
         # (if a TaskWrapper was passed in):
         galaxy_job_id = job_wrapper.get_id_tag()
-        log.debug("(%s) submitting file %s" % (galaxy_job_id, job_file))
+        log.debug("({}) submitting file {}".format(galaxy_job_id, job_file))
 
         tries = 0
         while tries < 5:
@@ -322,9 +320,9 @@ class PBSJobRunner(AsynchronousJobRunner):
             return
 
         if pbs_queue_name is None:
-            log.debug("(%s) queued in default queue as %s" % (galaxy_job_id, job_id))
+            log.debug("({}) queued in default queue as {}".format(galaxy_job_id, job_id))
         else:
-            log.debug("(%s) queued in %s queue as %s" % (galaxy_job_id, pbs_queue_name, job_id))
+            log.debug("({}) queued in {} queue as {}".format(galaxy_job_id, pbs_queue_name, job_id))
 
         # persist destination
         job_wrapper.set_job_destination(job_destination, job_id)
@@ -358,7 +356,7 @@ class PBSJobRunner(AsynchronousJobRunner):
             old_state = pbs_job_state.old_state
             pbs_server_name = self.__get_pbs_server(pbs_job_state.job_destination.params)
             if pbs_server_name in failures:
-                log.debug("(%s/%s) Skipping state check because PBS server connection failed" % (galaxy_job_id, job_id))
+                log.debug("({}/{}) Skipping state check because PBS server connection failed".format(galaxy_job_id, job_id))
                 new_watched.append(pbs_job_state)
                 continue
             try:
@@ -369,13 +367,13 @@ class PBSJobRunner(AsynchronousJobRunner):
                 try:
                     # Recheck to make sure it wasn't a communication problem
                     self.check_single_job(pbs_server_name, job_id)
-                    log.warning("(%s/%s) PBS job was not in state check list, but was found with individual state check" % (galaxy_job_id, job_id))
+                    log.warning("({}/{}) PBS job was not in state check list, but was found with individual state check".format(galaxy_job_id, job_id))
                     new_watched.append(pbs_job_state)
                 except Exception:
                     errno, text = pbs.error()
                     if errno == 15001:
                         # 15001 == job not in queue
-                        log.debug("(%s/%s) PBS job has left queue" % (galaxy_job_id, job_id))
+                        log.debug("({}/{}) PBS job has left queue".format(galaxy_job_id, job_id))
                         self.work_queue.put((self.finish_job, pbs_job_state))
                     else:
                         # Unhandled error, continue to monitor
@@ -383,7 +381,7 @@ class PBSJobRunner(AsynchronousJobRunner):
                         new_watched.append(pbs_job_state)
                 continue
             if status.job_state != old_state:
-                log.debug("(%s/%s) PBS job state changed from %s to %s" % (galaxy_job_id, job_id, old_state, status.job_state))
+                log.debug("({}/{}) PBS job state changed from {} to {}".format(galaxy_job_id, job_id, old_state, status.job_state))
             if status.job_state == "R" and not pbs_job_state.running:
                 pbs_job_state.running = True
                 pbs_job_state.job_wrapper.change_state(model.Job.states.RUNNING)
@@ -398,18 +396,18 @@ class PBSJobRunner(AsynchronousJobRunner):
                 # "keep_completed" is enabled in PBS, so try to check exit status
                 try:
                     assert int(status.exit_status) == 0
-                    log.debug("(%s/%s) PBS job has completed successfully" % (galaxy_job_id, job_id))
+                    log.debug("({}/{}) PBS job has completed successfully".format(galaxy_job_id, job_id))
                 except AssertionError:
                     exit_status = int(status.exit_status)
                     error_message = JOB_EXIT_STATUS.get(exit_status, 'Unknown error: %s' % status.exit_status)
                     pbs_job_state.fail_message = CLUSTER_ERROR_MESSAGE % error_message
-                    log.error('(%s/%s) PBS job failed: %s' % (galaxy_job_id, job_id, error_message))
+                    log.error('({}/{}) PBS job failed: {}'.format(galaxy_job_id, job_id, error_message))
                     pbs_job_state.stop_job = False
                     self.work_queue.put((self.fail_job, pbs_job_state))
                     continue
                 except AttributeError:
                     # No exit_status, can't verify proper completion so we just have to assume success.
-                    log.debug("(%s/%s) PBS job has completed" % (galaxy_job_id, job_id))
+                    log.debug("({}/{}) PBS job has completed".format(galaxy_job_id, job_id))
                 self.work_queue.put((self.finish_job, pbs_job_state))
                 continue
             pbs_job_state.old_state = status.job_state
@@ -499,14 +497,14 @@ class PBSJobRunner(AsynchronousJobRunner):
                     stage_name = os.path.join(self.app.config.pbs_stage_path, os.path.split(fname)[1])
                 else:
                     stage_name = fname
-                stage += "%s@%s:%s" % (stage_name, self.app.config.pbs_dataset_server, fname)
+                stage += "{}@{}:{}".format(stage_name, self.app.config.pbs_dataset_server, fname)
         return stage
 
     def stop_job(self, job_wrapper):
         """Attempts to delete a job from the PBS queue"""
         job = job_wrapper.get_job()
         job_id = job.get_job_runner_external_id().encode('utf-8')
-        job_tag = "(%s/%s)" % (job.get_id_tag(), job_id)
+        job_tag = "({}/{})".format(job.get_id_tag(), job_id)
         log.debug("%s Stopping PBS job" % job_tag)
 
         # Declare the connection handle c so that it can be cleaned up:
@@ -528,7 +526,7 @@ class PBSJobRunner(AsynchronousJobRunner):
                       % job_tag)
         except Exception:
             e = traceback.format_exc()
-            log.debug("%s Unable to stop job: %s" % (job_tag, e))
+            log.debug("{} Unable to stop job: {}".format(job_tag, e))
         finally:
             # Cleanup: disconnect from the server.
             if (None is not c):
@@ -538,22 +536,22 @@ class PBSJobRunner(AsynchronousJobRunner):
         """Recovers jobs stuck in the queued/running state when Galaxy started"""
         job_id = job.get_job_runner_external_id()
         pbs_job_state = AsynchronousJobState()
-        pbs_job_state.output_file = "%s/%s.o" % (self.app.config.cluster_files_directory, job.id)
-        pbs_job_state.error_file = "%s/%s.e" % (self.app.config.cluster_files_directory, job.id)
-        pbs_job_state.exit_code_file = "%s/%s.ec" % (self.app.config.cluster_files_directory, job.id)
-        pbs_job_state.job_file = "%s/%s.sh" % (self.app.config.cluster_files_directory, job.id)
+        pbs_job_state.output_file = "{}/{}.o".format(self.app.config.cluster_files_directory, job.id)
+        pbs_job_state.error_file = "{}/{}.e".format(self.app.config.cluster_files_directory, job.id)
+        pbs_job_state.exit_code_file = "{}/{}.ec".format(self.app.config.cluster_files_directory, job.id)
+        pbs_job_state.job_file = "{}/{}.sh".format(self.app.config.cluster_files_directory, job.id)
         pbs_job_state.job_id = str(job_id)
         pbs_job_state.runner_url = job_wrapper.get_job_runner_url()
         pbs_job_state.job_destination = job_wrapper.job_destination
         job_wrapper.command_line = job.command_line
         pbs_job_state.job_wrapper = job_wrapper
         if job.state == model.Job.states.RUNNING:
-            log.debug("(%s/%s) is still in running state, adding to the PBS queue" % (job.id, job.get_job_runner_external_id()))
+            log.debug("({}/{}) is still in running state, adding to the PBS queue".format(job.id, job.get_job_runner_external_id()))
             pbs_job_state.old_state = 'R'
             pbs_job_state.running = True
             self.monitor_queue.put(pbs_job_state)
         elif job.state == model.Job.states.QUEUED:
-            log.debug("(%s/%s) is still in PBS queued state, adding to the PBS queue" % (job.id, job.get_job_runner_external_id()))
+            log.debug("({}/{}) is still in PBS queued state, adding to the PBS queue".format(job.id, job.get_job_runner_external_id()))
             pbs_job_state.old_state = 'Q'
             pbs_job_state.running = False
             self.monitor_queue.put(pbs_job_state)
