@@ -112,7 +112,7 @@ class UnivaJobRunner(DRMAAJobRunner):
             log.error("DRMAAUniva: job {job_id} determined unknown state {state}".format(job_id=ajs.job_id, state=state))
             drmaa_state = self.drmaa_job_states.FAILED
         # by default, finish the job with the state from drmaa
-        return super(UnivaJobRunner, self)._complete_terminal_job(ajs, drmaa_state=drmaa_state)
+        return super()._complete_terminal_job(ajs, drmaa_state=drmaa_state)
 
     def _drmaa_state_is_refined(self, statep, staten):
         """
@@ -192,8 +192,8 @@ class UnivaJobRunner(DRMAAJobRunner):
         - FAILED if failed not in [0,24,25,100]
         '''
         # log.debug("UnivaJobRunner._get_drmaa_state_qacct ({jobid})".format(jobid=job_id))
-        signals = dict((k, v) for v, k in reversed(sorted(signal.__dict__.items()))
-           if v.startswith('SIG') and not v.startswith('SIG_'))
+        signals = {k: v for v, k in reversed(sorted(signal.__dict__.items()))
+           if v.startswith('SIG') and not v.startswith('SIG_')}
         cmd = ['qacct', '-j', job_id]
         slp = 1
         # run qacct -j JOBID (since the accounting data for the job might not be
@@ -422,13 +422,15 @@ class UnivaJobRunner(DRMAAJobRunner):
         # log.debug("UnivaJobRunner._get_drmaa_state_wait ({jobid}) -> {state}".format(jobid=job_id, state=self.drmaa_job_state_strings[state]))
         return state
 
-    def _get_drmaa_state(self, job_id, ds, waitqacct, extinfo=dict()):
+    def _get_drmaa_state(self, job_id, ds, waitqacct, extinfo=None):
         """
         get the state using drmaa.job_info/qstat and drmaa.wait/qacct using the above functions
         qacct/wait is only called if waitqacct is True.
         the function returns the state (one of the drmaa states) and extended
         information in the extinfo dict
         """
+        if extinfo is None:
+            extinfo = {}
         # log.debug("UnivaJobRunner._get_drmaa_state ({jobid}) {qw}".format(jobid=job_id, qw=waitqacct))
         state = self.drmaa.JobState.UNDETERMINED
         # try to get the state with drmaa.job_status (does not work for jobs
