@@ -1,6 +1,4 @@
-
-
-class ApiKeyManager(object):
+class ApiKeyManager:
 
     def __init__(self, app):
         self.app = app
