@@ -1,8 +1,8 @@
-from __future__ import absolute_import
-
 import logging
 from collections import OrderedDict
 
+from sqlalchemy.orm import joinedload, Query
+
 from galaxy import model
 from galaxy.exceptions import (
     ItemAccessibilityException,
@@ -30,7 +30,7 @@ ERROR_INVALID_ELEMENTS_SPECIFICATION = "Create called with invalid parameters, m
 ERROR_NO_COLLECTION_TYPE = "Create called without specifying a collection type."
 
 
-class DatasetCollectionManager(object):
+class DatasetCollectionManager:
     """
     Abstraction for interfacing with dataset collections instance - ideally abstracts
     out model and plugin details.
@@ -48,15 +48,15 @@ class DatasetCollectionManager(object):
         self.tag_handler = tags.GalaxyTagHandler(app.model.context)
         self.ldda_manager = lddas.LDDAManager(app)
 
-    def precreate_dataset_collection_instance(self, trans, parent, name, structure, implicit_inputs=None, implicit_output_name=None, tags=None):
+    def precreate_dataset_collection_instance(self, trans, parent, name, structure, implicit_inputs=None, implicit_output_name=None, tags=None, completed_collection=None):
         # TODO: prebuild all required HIDs and send them in so no need to flush in between.
-        dataset_collection = self.precreate_dataset_collection(structure, allow_unitialized_element=implicit_output_name is not None)
+        dataset_collection = self.precreate_dataset_collection(structure, allow_unitialized_element=implicit_output_name is not None, completed_collection=completed_collection, implicit_output_name=implicit_output_name)
         instance = self._create_instance_for_collection(
             trans, parent, name, dataset_collection, implicit_inputs=implicit_inputs, implicit_output_name=implicit_output_name, flush=False, tags=tags
         )
         return instance
 
-    def precreate_dataset_collection(self, structure, allow_unitialized_element=True):
+    def precreate_dataset_collection(self, structure, allow_unitialized_element=True, completed_collection=None, implicit_output_name=None):
         has_structure = not structure.is_leaf and structure.children_known
         if not has_structure and allow_unitialized_element:
             dataset_collection = model.DatasetCollectionElement.UNINITIALIZED_ELEMENT
@@ -71,10 +71,17 @@ class DatasetCollectionManager(object):
             elements = []
             for index, (identifier, substructure) in enumerate(structure.children):
                 # TODO: Open question - populate these now or later?
-                if substructure.is_leaf:
-                    element = model.DatasetCollectionElement.UNINITIALIZED_ELEMENT
-                else:
-                    element = self.precreate_dataset_collection(substructure, allow_unitialized_element=allow_unitialized_element)
+                element = None
+                if completed_collection and implicit_output_name:
+                    job = completed_collection[index]
+                    if job:
+                        it = (jtiodca.dataset_collection for jtiodca in job.output_dataset_collections if jtiodca.name == implicit_output_name)
+                        element = next(it, None)
+                if element is None:
+                    if substructure.is_leaf:
+                        element = model.DatasetCollectionElement.UNINITIALIZED_ELEMENT
+                    else:
+                        element = self.precreate_dataset_collection(substructure, allow_unitialized_element=allow_unitialized_element)
 
                 element = model.DatasetCollectionElement(
                     element=element,
@@ -89,7 +96,8 @@ class DatasetCollectionManager(object):
 
     def create(self, trans, parent, name, collection_type, element_identifiers=None,
                elements=None, implicit_collection_info=None, trusted_identifiers=None,
-               hide_source_items=False, tags=None, copy_elements=False, history=None):
+               hide_source_items=False, tags=None, copy_elements=False, history=None,
+               set_hid=True, flush=True, completed_job=None, output_name=None):
         """
         PRECONDITION: security checks on ability to add to parent
         occurred during load.
@@ -101,15 +109,19 @@ class DatasetCollectionManager(object):
         if element_identifiers and not trusted_identifiers:
             validate_input_element_identifiers(element_identifiers)
 
-        dataset_collection = self.create_dataset_collection(
-            trans=trans,
-            collection_type=collection_type,
-            element_identifiers=element_identifiers,
-            elements=elements,
-            hide_source_items=hide_source_items,
-            copy_elements=copy_elements,
-            history=history,
-        )
+        if completed_job and output_name:
+            jtodca = next(a for a in completed_job.output_dataset_collection_instances if a.name == output_name)
+            dataset_collection = jtodca.dataset_collection_instance.collection
+        else:
+            dataset_collection = self.create_dataset_collection(
+                trans=trans,
+                collection_type=collection_type,
+                element_identifiers=element_identifiers,
+                elements=elements,
+                hide_source_items=hide_source_items,
+                copy_elements=copy_elements,
+                history=history,
+            )
 
         implicit_inputs = []
         if implicit_collection_info:
@@ -120,10 +132,10 @@ class DatasetCollectionManager(object):
             implicit_output_name = implicit_collection_info["implicit_output_name"]
 
         return self._create_instance_for_collection(
-            trans, parent, name, dataset_collection, implicit_inputs=implicit_inputs, implicit_output_name=implicit_output_name, tags=tags
+            trans, parent, name, dataset_collection, implicit_inputs=implicit_inputs, implicit_output_name=implicit_output_name, tags=tags, set_hid=set_hid, flush=flush,
         )
 
-    def _create_instance_for_collection(self, trans, parent, name, dataset_collection, implicit_output_name=None, implicit_inputs=None, tags=None, flush=True):
+    def _create_instance_for_collection(self, trans, parent, name, dataset_collection, implicit_output_name=None, implicit_inputs=None, tags=None, set_hid=True, flush=True):
         if isinstance(parent, model.History):
             dataset_collection_instance = self.model.HistoryDatasetCollectionAssociation(
                 collection=dataset_collection,
@@ -137,8 +149,9 @@ class DatasetCollectionManager(object):
                 dataset_collection_instance.implicit_output_name = implicit_output_name
 
             log.debug("Created collection with %d elements" % (len(dataset_collection_instance.collection.elements)))
-            # Handle setting hid
-            parent.add_dataset_collection(dataset_collection_instance)
+
+            if set_hid:
+                parent.add_dataset_collection(dataset_collection_instance)
 
         elif isinstance(parent, model.LibraryFolder):
             dataset_collection_instance = self.model.LibraryDatasetCollectionAssociation(
@@ -562,3 +575,15 @@ class DatasetCollectionManager(object):
             if not trans.app.security_agent.can_access_library_item(trans.get_current_user_roles(), collection_instance, trans.user):
                 raise ItemAccessibilityException("LibraryDatasetCollectionAssociation is not accessible to the current user", type='error')
         return collection_instance
+
+    def get_collection_contents_qry(self, parent_id, limit=None, offset=None):
+        """Find first level of collection contents by containing collection parent_id"""
+        DCE = model.DatasetCollectionElement
+        qry = Query(DCE).filter(DCE.dataset_collection_id == parent_id)
+        qry = qry.order_by(DCE.element_index)
+        qry = qry.options(joinedload('child_collection'), joinedload('hda'))
+        if limit is not None:
+            qry = qry.limit(int(limit))
+        if offset is not None:
+            qry = qry.offset(int(offset))
+        return qry
