@@ -55,7 +55,7 @@ class UserManager(base.ModelManager, deletable.PurgableManagerMixin):
     # TODO: incorporate UsesFormDefinitionsMixin?
     def __init__(self, app):
         self.model_class = app.model.User
-        super(UserManager, self).__init__(app)
+        super().__init__(app)
 
     def register(self, trans, email=None, username=None, password=None, confirm=None, subscribe=False):
         """
@@ -116,7 +116,7 @@ class UserManager(base.ModelManager, deletable.PurgableManagerMixin):
         """Mark the given user deleted."""
         if not self.app.config.allow_user_deletion:
             raise exceptions.ConfigDoesNotAllowException('The configuration of this Galaxy instance does not allow admins to delete users.')
-        super(UserManager, self).delete(user, flush=flush)
+        super().delete(user, flush=flush)
 
     def undelete(self, user, flush=True):
         """Remove the deleted flag for the given user."""
@@ -124,7 +124,7 @@ class UserManager(base.ModelManager, deletable.PurgableManagerMixin):
             raise exceptions.ConfigDoesNotAllowException('The configuration of this Galaxy instance does not allow admins to undelete users.')
         if user.purged:
             raise exceptions.ItemDeletionException('Purged user cannot be undeleted.')
-        super(UserManager, self).undelete(user, flush=flush)
+        super().undelete(user, flush=flush)
 
     def purge(self, user, flush=True):
         """Purge the given user. They must have the deleted flag already."""
@@ -197,7 +197,7 @@ class UserManager(base.ModelManager, deletable.PurgableManagerMixin):
                 addr.phone = new_secure_hash(addr.phone + pseudorandom_value)
                 self.session().add(addr)
         # Purge the user
-        super(UserManager, self).purge(user, flush=flush)
+        super().purge(user, flush=flush)
 
     def _error_on_duplicate_email(self, email):
         """
@@ -217,7 +217,7 @@ class UserManager(base.ModelManager, deletable.PurgableManagerMixin):
         filters = self._munge_filters(self.model_class.email == email, filters)
         try:
             # TODO: use one_or_none
-            return super(UserManager, self).one(filters=filters, **kwargs)
+            return super().one(filters=filters, **kwargs)
         except exceptions.ObjectNotFound:
             return None
 
@@ -227,7 +227,7 @@ class UserManager(base.ModelManager, deletable.PurgableManagerMixin):
         """
         filters = self._munge_filters(self.model_class.email.like(email_with_wildcards), filters)
         order_by = order_by or (model.User.email, )
-        return super(UserManager, self).list(filters=filters, order_by=order_by, **kwargs)
+        return super().list(filters=filters, order_by=order_by, **kwargs)
 
     # ---- admin
     def is_admin(self, user, trans=None):
@@ -248,7 +248,7 @@ class UserManager(base.ModelManager, deletable.PurgableManagerMixin):
         """
         admin_emails = self.app.config.admin_users_list
         filters = self._munge_filters(self.model_class.email.in_(admin_emails), filters)
-        return super(UserManager, self).list(filters=filters, **kwargs)
+        return super().list(filters=filters, **kwargs)
 
     def error_unless_admin(self, user, msg="Administrators only", **kwargs):
         """
@@ -318,7 +318,7 @@ class UserManager(base.ModelManager, deletable.PurgableManagerMixin):
 
     # ---- preferences
     def preferences(self, user):
-        return dict((key, value) for key, value in user.preferences.items())
+        return {key: value for key, value in user.preferences.items()}
 
     # ---- roles and permissions
     def private_role(self, user):
@@ -461,7 +461,7 @@ class UserManager(base.ModelManager, deletable.PurgableManagerMixin):
             trans.sa_session.flush()
         return activation_token
 
-    def send_reset_email(self, trans, payload={}, **kwd):
+    def send_reset_email(self, trans, payload, **kwd):
         """Reset the user's password. Send an email with token that allows a password change."""
         if self.app.config.smtp_server is None:
             return "Mail is not configured for this Galaxy instance and password reset information cannot be sent. Please contact your local Galaxy administrator."
@@ -535,7 +535,7 @@ class UserSerializer(base.ModelSerializer, deletable.PurgableSerializerMixin):
         """
         Convert a User and associated data to a dictionary representation.
         """
-        super(UserSerializer, self).__init__(app)
+        super().__init__(app)
         self.user_manager = self.manager
 
         self.default_view = 'summary'
@@ -562,7 +562,7 @@ class UserSerializer(base.ModelSerializer, deletable.PurgableSerializerMixin):
         ], include_keys_from='summary')
 
     def add_serializers(self):
-        super(UserSerializer, self).add_serializers()
+        super().add_serializers()
         deletable.PurgableSerializerMixin.add_serializers(self)
 
         self.serializers.update({
@@ -589,13 +589,13 @@ class UserDeserializer(base.ModelDeserializer):
     model_manager_class = UserManager
 
     def add_deserializers(self):
-        super(UserDeserializer, self).add_deserializers()
+        super().add_deserializers()
         self.deserializers.update({
             'username'  : self.deserialize_username,
         })
 
     def deserialize_username(self, item, key, username, trans=None, **context):
-        # TODO: validate_user_input requires trans and should(?) raise exceptions
+        # TODO: validate_publicname requires trans and should(?) raise exceptions
         # move validation to UserValidator and use self.app, exceptions instead
         validation_error = validate_publicname(trans, username, user=item)
         if validation_error:
@@ -645,7 +645,7 @@ class AdminUserFilterParser(base.ModelFilterParser, deletable.PurgableFiltersMix
     model_class = model.User
 
     def _add_parsers(self):
-        super(AdminUserFilterParser, self)._add_parsers()
+        super()._add_parsers()
         deletable.PurgableFiltersMixin._add_parsers(self)
 
         # PRECONDITION: user making the query has been verified as an admin
