@@ -24,7 +24,7 @@ from galaxy.util.tool_shed.xml_util import parse_xml
 log = logging.getLogger(__name__)
 
 
-class InstalledRepositoryManager(object):
+class InstalledRepositoryManager:
 
     def __init__(self, app):
         """
@@ -161,14 +161,14 @@ class InstalledRepositoryManager(object):
         repository_dependency_tups = self.get_repository_dependency_tups_for_installed_repository(repository, status=status)
         # Add an entry to self.installed_repository_dependencies_of_installed_repositories.
         if repository_tup not in self.installed_repository_dependencies_of_installed_repositories:
-            debug_msg = "Adding an entry for revision %s of repository %s owned by %s " % (installed_changeset_revision, name, owner)
+            debug_msg = "Adding an entry for revision {} of repository {} owned by {} ".format(installed_changeset_revision, name, owner)
             debug_msg += "to installed_repository_dependencies_of_installed_repositories."
             log.debug(debug_msg)
             self.installed_repository_dependencies_of_installed_repositories[repository_tup] = repository_dependency_tups
         # Use the repository_dependency_tups to add entries to the reverse dictionary
         # self.installed_dependent_repositories_of_installed_repositories.
         for required_repository_tup in repository_dependency_tups:
-            debug_msg = "Appending revision %s of repository %s owned by %s " % (installed_changeset_revision, name, owner)
+            debug_msg = "Appending revision {} of repository {} owned by {} ".format(installed_changeset_revision, name, owner)
             debug_msg += "to all dependent repositories in installed_dependent_repositories_of_installed_repositories."
             log.debug(debug_msg)
             if required_repository_tup in self.installed_dependent_repositories_of_installed_repositories:
@@ -181,7 +181,7 @@ class InstalledRepositoryManager(object):
         tool_dependency_tup = self.get_tool_dependency_tuple_for_installed_repository_manager(tool_dependency)
         if tool_dependency_tup not in self.installed_runtime_dependent_tool_dependencies_of_installed_tool_dependencies:
             tool_shed_repository_id, name, version, type = tool_dependency_tup
-            debug_msg = "Adding an entry for version %s of %s %s " % (version, type, name)
+            debug_msg = "Adding an entry for version {} of {} {} ".format(version, type, name)
             debug_msg += "to installed_runtime_dependent_tool_dependencies_of_installed_tool_dependencies."
             log.debug(debug_msg)
             status = self.install_model.ToolDependency.installation_status.INSTALLED
@@ -195,7 +195,7 @@ class InstalledRepositoryManager(object):
         repository_tup = self.get_repository_tuple_for_installed_repository_manager(repository)
         if repository_tup not in self.installed_tool_dependencies_of_installed_repositories:
             tool_shed, name, owner, installed_changeset_revision = repository_tup
-            debug_msg = "Adding an entry for revision %s of repository %s owned by %s " % (installed_changeset_revision, name, owner)
+            debug_msg = "Adding an entry for revision {} of repository {} owned by {} ".format(installed_changeset_revision, name, owner)
             debug_msg += "to installed_tool_dependencies_of_installed_repositories."
             log.debug(debug_msg)
             installed_tool_dependency_tups = []
@@ -210,7 +210,7 @@ class InstalledRepositoryManager(object):
         repository_tup = self.get_repository_tuple_for_installed_repository_manager(repository)
         if repository_tup not in self.repository_dependencies_of_installed_repositories:
             tool_shed, name, owner, installed_changeset_revision = repository_tup
-            debug_msg = "Adding an entry for revision %s of repository %s owned by %s " % (installed_changeset_revision, name, owner)
+            debug_msg = "Adding an entry for revision {} of repository {} owned by {} ".format(installed_changeset_revision, name, owner)
             debug_msg += "to repository_dependencies_of_installed_repositories."
             log.debug(debug_msg)
             repository_dependency_tups = self.get_repository_dependency_tups_for_installed_repository(repository, status=None)
@@ -221,7 +221,7 @@ class InstalledRepositoryManager(object):
         tool_dependency_tup = self.get_tool_dependency_tuple_for_installed_repository_manager(tool_dependency)
         if tool_dependency_tup not in self.runtime_tool_dependencies_of_installed_tool_dependencies:
             tool_shed_repository_id, name, version, type = tool_dependency_tup
-            debug_msg = "Adding an entry for version %s of %s %s " % (version, type, name)
+            debug_msg = "Adding an entry for version {} of {} {} ".format(version, type, name)
             debug_msg += "to runtime_tool_dependencies_of_installed_tool_dependencies."
             log.debug(debug_msg)
             runtime_dependent_tool_dependency_tups = self.get_runtime_dependent_tool_dependency_tuples(tool_dependency,
@@ -234,7 +234,7 @@ class InstalledRepositoryManager(object):
         repository_tup = self.get_repository_tuple_for_installed_repository_manager(repository)
         if repository_tup not in self.tool_dependencies_of_installed_repositories:
             tool_shed, name, owner, installed_changeset_revision = repository_tup
-            debug_msg = "Adding an entry for revision %s of repository %s owned by %s " % (installed_changeset_revision, name, owner)
+            debug_msg = "Adding an entry for revision {} of repository {} owned by {} ".format(installed_changeset_revision, name, owner)
             debug_msg += "to tool_dependencies_of_installed_repositories."
             log.debug(debug_msg)
             tool_dependency_tups = []
@@ -615,7 +615,7 @@ class InstalledRepositoryManager(object):
                     env_shell_file_path = td.get_env_shell_file_path(self.app)
                     if env_shell_file_path is not None:
                         try:
-                            contents = open(env_shell_file_path, 'r').read()
+                            contents = open(env_shell_file_path).read()
                         except Exception as e:
                             contents = None
                             log.debug('Error reading file %s, so cannot determine if package %s requires package %s at run time: %s' %
@@ -715,7 +715,7 @@ class InstalledRepositoryManager(object):
                 log.debug("Removed repository installation directory: %s" % str(repository_install_dir))
                 removed = True
             except Exception as e:
-                log.debug("Error removing repository installation directory %s: %s" % (str(repository_install_dir), str(e)))
+                log.debug("Error removing repository installation directory {}: {}".format(str(repository_install_dir), str(e)))
                 if isinstance(e, OSError) and not os.path.exists(repository_install_dir):
                     removed = True
                     log.debug("Repository directory does not exist on disk, marking as uninstalled.")
@@ -730,7 +730,7 @@ class InstalledRepositoryManager(object):
                 for tool_dependency in tool_dependencies_to_uninstall:
                     uninstalled, error_message = tool_dependency_util.remove_tool_dependency(self.app, tool_dependency)
                     if error_message:
-                        errors = '%s  %s' % (errors, error_message)
+                        errors = '{}  {}'.format(errors, error_message)
         repository.deleted = True
         if remove_from_disk:
             repository.status = self.app.install_model.ToolShedRepository.installation_status.UNINSTALLED
@@ -878,7 +878,7 @@ class InstalledRepositoryManager(object):
             altered_installed_dependent_repositories_of_installed_repositories
         # Remove this repository's entry from self.installed_repository_dependencies_of_installed_repositories.
         if repository_tup in self.installed_repository_dependencies_of_installed_repositories:
-            debug_msg = "Removing entry for revision %s of repository %s owned by %s " % (installed_changeset_revision, name, owner)
+            debug_msg = "Removing entry for revision {} of repository {} owned by {} ".format(installed_changeset_revision, name, owner)
             debug_msg += "from installed_repository_dependencies_of_installed_repositories."
             log.debug(debug_msg)
             del self.installed_repository_dependencies_of_installed_repositories[repository_tup]
@@ -888,7 +888,7 @@ class InstalledRepositoryManager(object):
         tool_dependency_tup = self.get_tool_dependency_tuple_for_installed_repository_manager(tool_dependency)
         if tool_dependency_tup in self.installed_runtime_dependent_tool_dependencies_of_installed_tool_dependencies:
             tool_shed_repository_id, name, version, type = tool_dependency_tup
-            debug_msg = "Removing entry for version %s of %s %s " % (version, type, name)
+            debug_msg = "Removing entry for version {} of {} {} ".format(version, type, name)
             debug_msg += "from installed_runtime_dependent_tool_dependencies_of_installed_tool_dependencies."
             log.debug(debug_msg)
             del self.installed_runtime_dependent_tool_dependencies_of_installed_tool_dependencies[tool_dependency_tup]
@@ -898,7 +898,7 @@ class InstalledRepositoryManager(object):
         repository_tup = self.get_repository_tuple_for_installed_repository_manager(repository)
         if repository_tup in self.installed_tool_dependencies_of_installed_repositories:
             tool_shed, name, owner, installed_changeset_revision = repository_tup
-            debug_msg = "Removing entry for revision %s of repository %s owned by %s " % (installed_changeset_revision, name, owner)
+            debug_msg = "Removing entry for revision {} of repository {} owned by {} ".format(installed_changeset_revision, name, owner)
             debug_msg += "from installed_tool_dependencies_of_installed_repositories."
             log.debug(debug_msg)
             del self.installed_tool_dependencies_of_installed_repositories[repository_tup]
@@ -908,7 +908,7 @@ class InstalledRepositoryManager(object):
         repository_tup = self.get_repository_tuple_for_installed_repository_manager(repository)
         if repository_tup in self.repository_dependencies_of_installed_repositories:
             tool_shed, name, owner, installed_changeset_revision = repository_tup
-            debug_msg = "Removing entry for revision %s of repository %s owned by %s " % (installed_changeset_revision, name, owner)
+            debug_msg = "Removing entry for revision {} of repository {} owned by {} ".format(installed_changeset_revision, name, owner)
             debug_msg += "from repository_dependencies_of_installed_repositories."
             log.debug(debug_msg)
             del self.repository_dependencies_of_installed_repositories[repository_tup]
