@@ -17,7 +17,7 @@ from galaxy.util.tool_shed.xml_util import parse_xml
 log = logging.getLogger(__name__)
 
 
-class ToolPanelManager(object):
+class ToolPanelManager:
 
     def __init__(self, app):
         self.app = app
@@ -34,13 +34,14 @@ class ToolPanelManager(object):
             return
         old_toolbox = self.app.toolbox
         shed_tool_conf = shed_tool_conf_dict['config_filename']
+        tool_cache_data_dir = shed_tool_conf_dict.get('tool_cache_data_dir')
         tool_path = shed_tool_conf_dict['tool_path']
         config_elems = []
         # Ideally shed_tool_conf.xml would be created before the repo is cloned and added to the DB, but this is called
         # from too many places to make it feasible at this time
         try:
             tree, error_message = parse_xml(shed_tool_conf, check_exists=False)
-        except (OSError, IOError) as exc:
+        except OSError as exc:
             if (exc.errno == errno.ENOENT and shed_tool_conf_dict.get('create', None) is not None):
                 log.info('Creating shed tool config with default contents: %s', shed_tool_conf)
                 with open(shed_tool_conf, 'w') as fh:
@@ -57,7 +58,7 @@ class ToolPanelManager(object):
             for elem_entry in elem_list:
                 config_elems.append(elem_entry)
             # Persist the altered shed_tool_config file.
-            self.config_elems_to_xml_file(config_elems, shed_tool_conf, tool_path)
+            self.config_elems_to_xml_file(config_elems, shed_tool_conf, tool_path, tool_cache_data_dir)
             self.app.wait_for_toolbox_reload(old_toolbox)
         else:
             log.error(error_message)
@@ -94,13 +95,14 @@ class ToolPanelManager(object):
             self.app.toolbox.update_shed_config(shed_tool_conf_dict)
             self.add_to_shed_tool_config(shed_tool_conf_dict, elem_list)
 
-    def config_elems_to_xml_file(self, config_elems, config_filename, tool_path):
+    def config_elems_to_xml_file(self, config_elems, config_filename, tool_path, tool_cache_data_dir=None):
         """
         Persist the current in-memory list of config_elems to a file named by the
         value of config_filename.
         """
         try:
-            root = parse_xml_string('<?xml version="1.0"?>\n<toolbox tool_path="%s"></toolbox>' % str(tool_path))
+            tool_cache_data_dir = ' tool_cache_data_dir="{}"'.format(tool_cache_data_dir) if tool_cache_data_dir else ''
+            root = parse_xml_string('<?xml version="1.0"?>\n<toolbox tool_path="{}"{}></toolbox>'.format(tool_path, tool_cache_data_dir))
             for elem in config_elems:
                 root.append(elem)
             with RenamedTemporaryFile(config_filename, mode='w') as fh:
