@@ -4,7 +4,6 @@ import logging
 import os
 from collections import OrderedDict
 
-from six import string_types
 
 from galaxy import util
 from galaxy.tools.data import TabularToolDataTable
@@ -17,7 +16,7 @@ VALUE_TRANSLATION_FUNCTIONS = dict(abspath=os.path.abspath)
 DEFAULT_VALUE_TRANSLATION_TYPE = 'template'
 
 
-class DataManagers(object):
+class DataManagers:
     def __init__(self, app, xml_filename=None):
         self.app = app
         self.data_managers = OrderedDict()
@@ -32,19 +31,19 @@ class DataManagers(object):
         if self.app.config.shed_data_manager_config_file:
             try:
                 self.load_from_xml(self.app.config.shed_data_manager_config_file, store_tool_path=True)
-            except (OSError, IOError) as exc:
-                if exc.errno != errno.ENOENT or self.app.config.shed_data_manager_config_file_set:
+            except OSError as exc:
+                if exc.errno != errno.ENOENT or self.app.config.is_set('shed_data_manager_config_file'):
                     raise
 
     def load_from_xml(self, xml_filename, store_tool_path=True):
         try:
             tree = util.parse_xml(xml_filename)
-        except (IOError, OSError) as e:
-            if e.errno != errno.ENOENT or self.app.config.data_manager_config_file_set:
+        except OSError as e:
+            if e.errno != errno.ENOENT or self.app.config.is_set('data_manager_config_file'):
                 raise
             return  # default config option and it doesn't exist, which is fine
         except Exception as e:
-            log.error('There was an error parsing your Data Manager config file "%s": %s' % (xml_filename, e))
+            log.error('There was an error parsing your Data Manager config file "{}": {}'.format(xml_filename, e))
             return  # we are not able to load any data managers
         root = tree.getroot()
         if root.tag != 'data_managers':
@@ -67,7 +66,7 @@ class DataManagers(object):
     def load_manager_from_elem(self, data_manager_elem, tool_path=None, add_manager=True):
         try:
             data_manager = DataManager(self, data_manager_elem, tool_path=tool_path)
-        except IOError as e:
+        except OSError as e:
             if e.errno == errno.ENOENT:
                 # File does not exist
                 return None
@@ -112,7 +111,7 @@ class DataManagers(object):
                         del self.managed_data_tables[data_table_name]
 
 
-class DataManager(object):
+class DataManager:
     GUID_TYPE = 'data_manager'
     DEFAULT_VERSION = "0.0.1"
 
@@ -255,7 +254,7 @@ class DataManager(object):
             try:
                 output_dict = json.loads(open(output_dataset.file_name).read())
             except Exception as e:
-                log.warning('Error reading DataManagerTool json for "%s": %s' % (output_name, e))
+                log.warning('Error reading DataManagerTool json for "{}": {}'.format(output_name, e))
                 continue
             data_manager_dicts[output_name] = output_dict
             for key, value in output_dict.items():
@@ -268,7 +267,7 @@ class DataManager(object):
         for data_table_name in self.data_tables.keys():
             data_table_values = data_tables_dict.pop(data_table_name, None)
             if not data_table_values:
-                log.warning('No values for data table "%s" were returned by the data manager "%s".' % (data_table_name, self.id))
+                log.warning('No values for data table "{}" were returned by the data manager "{}".'.format(data_table_name, self.id))
                 continue  # next data table
             data_table_remove_values = None
             if isinstance(data_table_values, dict):
@@ -280,10 +279,10 @@ class DataManager(object):
 
             data_table = self.data_managers.app.tool_data_tables.get(data_table_name, None)
             if data_table is None:
-                log.error('The data manager "%s" returned an unknown data table "%s" with new entries "%s". These entries will not be created. Please confirm that an entry for "%s" exists in your "%s" file.' % (self.id, data_table_name, data_table_values, data_table_name, 'tool_data_table_conf.xml'))
+                log.error('The data manager "{}" returned an unknown data table "{}" with new entries "{}". These entries will not be created. Please confirm that an entry for "{}" exists in your "{}" file.'.format(self.id, data_table_name, data_table_values, data_table_name, 'tool_data_table_conf.xml'))
                 continue  # next table name
             if not isinstance(data_table, SUPPORTED_DATA_TABLE_TYPES):
-                log.error('The data manager "%s" returned an unsupported data table "%s" with type "%s" with new entries "%s". These entries will not be created. Please confirm that the data table is of a supported type (%s).' % (self.id, data_table_name, type(data_table), data_table_values, SUPPORTED_DATA_TABLE_TYPES))
+                log.error('The data manager "{}" returned an unsupported data table "{}" with type "{}" with new entries "{}". These entries will not be created. Please confirm that the data table is of a supported type ({}).'.format(self.id, data_table_name, type(data_table), data_table_values, SUPPORTED_DATA_TABLE_TYPES))
                 continue  # next table name
             output_ref_values = {}
             if data_table_name in self.output_ref_by_data_table:
@@ -340,7 +339,7 @@ class DataManager(object):
             for data_table_name, data_table_values in data_tables_dict.items():
                 # tool returned extra data table entries, but data table was not declared in data manager
                 # do not add these values, but do provide messages
-                log.warning('The data manager "%s" returned an undeclared data table "%s" with new entries "%s". These entries will not be created. Please confirm that an entry for "%s" exists in your "%s" file.' % (self.id, data_table_name, data_table_values, data_table_name, self.data_managers.filename))
+                log.warning('The data manager "{}" returned an undeclared data table "{}" with new entries "{}". These entries will not be created. Please confirm that an entry for "{}" exists in your "{}" file.'.format(self.id, data_table_name, data_table_values, data_table_name, self.data_managers.filename))
 
     def process_move(self, data_table_name, column_name, source_base_path, relative_symlinks=False, **kwd):
         if data_table_name in self.move_by_data_table_column and column_name in self.move_by_data_table_column[data_table_name]:
@@ -381,7 +380,7 @@ class DataManager(object):
         value = kwd.get(column_name)
         if data_table_name in self.value_translation_by_data_table_column and column_name in self.value_translation_by_data_table_column[data_table_name]:
             for value_translation in self.value_translation_by_data_table_column[data_table_name][column_name]:
-                if isinstance(value_translation, string_types):
+                if isinstance(value_translation, str):
                     value = fill_template(value_translation, GALAXY_DATA_MANAGER_DATA_PATH=self.data_managers.app.config.galaxy_data_manager_data_path, **kwd).strip()
                 else:
                     value = value_translation(value)
