@@ -44,9 +44,9 @@ def execute(trans, tool, mapping_params, history, rerun_remap_job_id=None, colle
     )
 
     if invocation_step is None:
-        execution_tracker = ToolExecutionTracker(trans, tool, mapping_params, collection_info)
+        execution_tracker = ToolExecutionTracker(trans, tool, mapping_params, collection_info, completed_jobs=completed_jobs)
     else:
-        execution_tracker = WorkflowStepExecutionTracker(trans, tool, mapping_params, collection_info, invocation_step, job_callback=job_callback)
+        execution_tracker = WorkflowStepExecutionTracker(trans, tool, mapping_params, collection_info, invocation_step, job_callback=job_callback, completed_jobs=completed_jobs)
     execution_cache = ToolExecutionCache(trans)
 
     def execute_single_job(execution_slice, completed_job):
@@ -68,7 +68,7 @@ def execute(trans, tool, mapping_params, history, rerun_remap_job_id=None, colle
             del params['__workflow_resource_params__']
         if validate_outputs:
             params['__validate_outputs__'] = True
-        job, result = tool.handle_single_execution(trans, rerun_remap_job_id, execution_slice, history, execution_cache, completed_job, collection_info)
+        job, result = tool.handle_single_execution(trans, rerun_remap_job_id, execution_slice, history, execution_cache, completed_job, collection_info, flush_job=False)
         if job:
             log.debug(job_timer.to_str(tool_id=tool.id, job_id=job.id))
             execution_tracker.record_success(execution_slice, job, result)
@@ -93,6 +93,7 @@ def execute(trans, tool, mapping_params, history, rerun_remap_job_id=None, colle
 
     jobs_executed = 0
     has_remaining_jobs = False
+    datasets_to_persist = []
 
     for i, execution_slice in enumerate(execution_tracker.new_execution_slices()):
         if max_num_jobs and jobs_executed >= max_num_jobs:
@@ -100,6 +101,20 @@ def execute(trans, tool, mapping_params, history, rerun_remap_job_id=None, colle
             break
         else:
             execute_single_job(execution_slice, completed_jobs[i])
+            if execution_slice.datasets_to_persist:
+                datasets_to_persist.extend(execution_slice.datasets_to_persist)
+
+    if datasets_to_persist:
+        execution_slice.history.add_datasets(trans.sa_session, datasets_to_persist, set_hid=True, quota=False, flush=False)
+        # a side effect of history.add_datasets is a commit within db_next_hid (even with flush=False).
+    else:
+        # Make sure collections, implicit jobs etc are flushed even if there are no precreated output datasets
+        trans.sa_session.flush()
+    for job in execution_tracker.successful_jobs:
+        # Put the job in the queue if tracking in memory
+        tool.app.job_manager.enqueue(job, tool=tool, flush=False)
+        trans.log_event("Added job to the job queue, id: %s" % str(job.id), tool_id=job.tool_id)
+    trans.sa_session.flush()
 
     if has_remaining_jobs:
         raise PartialJobExecution(execution_tracker)
@@ -110,22 +125,25 @@ def execute(trans, tool, mapping_params, history, rerun_remap_job_id=None, colle
     return execution_tracker
 
 
-class ExecutionSlice(object):
+class ExecutionSlice:
 
     def __init__(self, job_index, param_combination, dataset_collection_elements=None):
         self.job_index = job_index
         self.param_combination = param_combination
         self.dataset_collection_elements = dataset_collection_elements
+        self.datasets_to_persist = None
+        self.history = None
 
 
-class ExecutionTracker(object):
+class ExecutionTracker:
 
-    def __init__(self, trans, tool, mapping_params, collection_info):
+    def __init__(self, trans, tool, mapping_params, collection_info, completed_jobs=None):
         # Known ahead of time...
         self.trans = trans
         self.tool = tool
         self.mapping_params = mapping_params
         self.collection_info = collection_info
+        self.completed_jobs = completed_jobs
 
         self._on_text = None
 
@@ -187,7 +205,7 @@ class ExecutionTracker(object):
                 job_params=None,
             )
         except Exception:
-            output_collection_name = "%s across %s" % (self.tool.name, on_text)
+            output_collection_name = "{} across {}".format(self.tool.name, on_text)
 
         return output_collection_name
 
@@ -283,6 +301,7 @@ class ExecutionTracker(object):
                 structure=effective_structure,
                 implicit_inputs=implicit_inputs,
                 implicit_output_name=output_name,
+                completed_collection=self.completed_jobs,
             )
             collection_instance.implicit_collection_jobs = implicit_collection_jobs
             collection_instances[output_name] = collection_instance
@@ -296,7 +315,7 @@ class ExecutionTracker(object):
     def implicit_collection_jobs(self):
         # TODO: refactor to track this properly maybe?
         if self.implicit_collections:
-            return six.next(six.itervalues(self.implicit_collections)).implicit_collection_jobs
+            return next(iter(self.implicit_collections.values())).implicit_collection_jobs
         else:
             return None
 
@@ -342,8 +361,7 @@ class ExecutionTracker(object):
             for job_index, param_combination in enumerate(self.param_combinations):
                 yield ExecutionSlice(job_index, param_combination)
         else:
-            for execution_slice in self.new_collection_execution_slices():
-                yield execution_slice
+            yield from self.new_collection_execution_slices()
 
     def record_success(self, execution_slice, job, outputs):
         # TODO: successful_jobs need to be inserted in the correct place...
@@ -361,7 +379,7 @@ class ExecutionTracker(object):
             job_assoc = model.ImplicitCollectionJobsJobAssociation()
             job_assoc.order_index = execution_slice.job_index
             job_assoc.implicit_collection_jobs = implicit_collection_jobs
-            job_assoc.job_id = job.id
+            job_assoc.job = job
             self.trans.sa_session.add(job_assoc)
 
 
@@ -369,8 +387,8 @@ class ExecutionTracker(object):
 # in the database immediately and they can be recovered.
 class ToolExecutionTracker(ExecutionTracker):
 
-    def __init__(self, trans, tool, mapping_params, collection_info):
-        super(ToolExecutionTracker, self).__init__(trans, tool, mapping_params, collection_info)
+    def __init__(self, trans, tool, mapping_params, collection_info, completed_jobs=None):
+        super().__init__(trans, tool, mapping_params, collection_info, completed_jobs=completed_jobs)
 
         # New to track these things for tool output API response in the tool case,
         # in the workflow case we just write stuff to the database and forget about
@@ -378,7 +396,7 @@ class ToolExecutionTracker(ExecutionTracker):
         self.outputs_by_output_name = collections.defaultdict(list)
 
     def record_success(self, execution_slice, job, outputs):
-        super(ToolExecutionTracker, self).record_success(execution_slice, job, outputs)
+        super().record_success(execution_slice, job, outputs)
         for output_name, output_dataset in outputs:
             if ToolOutputCollectionPart.is_named_collection_part_name(output_name):
                 # Skip known collection outputs, these will be covered by
@@ -390,36 +408,39 @@ class ToolExecutionTracker(ExecutionTracker):
 
     def new_collection_execution_slices(self):
         for job_index, (param_combination, dataset_collection_elements) in enumerate(six.moves.zip(self.param_combinations, self.walk_implicit_collections())):
-            for dataset_collection_element in dataset_collection_elements.values():
-                assert dataset_collection_element.element_object is None
+            completed_job = self.completed_jobs and self.completed_jobs[job_index]
+            if not completed_job:
+                for dataset_collection_element in dataset_collection_elements.values():
+                    assert dataset_collection_element.element_object is None
 
             yield ExecutionSlice(job_index, param_combination, dataset_collection_elements)
 
 
 class WorkflowStepExecutionTracker(ExecutionTracker):
 
-    def __init__(self, trans, tool, mapping_params, collection_info, invocation_step, job_callback):
-        super(WorkflowStepExecutionTracker, self).__init__(trans, tool, mapping_params, collection_info)
+    def __init__(self, trans, tool, mapping_params, collection_info, invocation_step, job_callback, completed_jobs=None):
+        super().__init__(trans, tool, mapping_params, collection_info, completed_jobs=completed_jobs)
         self.invocation_step = invocation_step
         self.job_callback = job_callback
 
     def record_success(self, execution_slice, job, outputs):
-        super(WorkflowStepExecutionTracker, self).record_success(execution_slice, job, outputs)
-        if self.collection_info:
-            self.invocation_step.implicit_collection_jobs = self.implicit_collection_jobs
-        else:
+        super().record_success(execution_slice, job, outputs)
+        if not self.collection_info:
             self.invocation_step.job = job
         self.job_callback(job)
 
     def new_collection_execution_slices(self):
         for job_index, (param_combination, dataset_collection_elements) in enumerate(six.moves.zip(self.param_combinations, self.walk_implicit_collections())):
-            found_result = False
-            for dataset_collection_element in dataset_collection_elements.values():
-                if dataset_collection_element.element_object is not None:
-                    found_result = True
-                    break
-            if found_result:
-                continue
+            completed_job = self.completed_jobs and self.completed_jobs[job_index]
+            if not completed_job:
+                found_result = False
+                for dataset_collection_element in dataset_collection_elements.values():
+                    if dataset_collection_element.element_object is not None:
+                        found_result = True
+                        break
+                if found_result:
+                    continue
+
             yield ExecutionSlice(job_index, param_combination, dataset_collection_elements)
 
     def ensure_implicit_collections_populated(self, history, params):
@@ -436,6 +457,7 @@ class WorkflowStepExecutionTracker(ExecutionTracker):
                 assert hasattr(implicit_collection, "history_content_type")  # make sure it is an HDCA and not a DC
                 collections[output_assoc.output_name] = output_assoc.dataset_collection
             self.implicit_collections = collections
+        self.invocation_step.implicit_collection_jobs = self.implicit_collection_jobs
 
 
 __all__ = ('execute', )
