@@ -178,7 +178,7 @@ class AbstractToolBox(Dictifiable, ManagesIntegratedToolPanelMixin):
         log.info("Parsing the tool configuration %s" % config_filename)
         try:
             tool_conf_source = get_toolbox_parser(config_filename)
-        except (OSError, IOError) as exc:
+        except OSError as exc:
             dynamic_confs = (self.app.config.shed_tool_config_file, self.app.config.migrated_tools_config)
             if config_filename in dynamic_confs and exc.errno == errno.ENOENT:
                 log.info("Shed-enabled tool configuration file does not exist, but will be created on demand: %s",
@@ -218,6 +218,7 @@ class AbstractToolBox(Dictifiable, ManagesIntegratedToolPanelMixin):
         if parsing_shed_tool_conf:
             shed_tool_conf_dict = dict(config_filename=config_filename,
                                        tool_path=tool_path,
+                                       tool_cache_data_dir=tool_cache_data_dir,
                                        config_elems=config_elems)
             self._dynamic_tool_confs.append(shed_tool_conf_dict)
 
@@ -350,7 +351,7 @@ class AbstractToolBox(Dictifiable, ManagesIntegratedToolPanelMixin):
                     new_tool_id=tool_id,
                     tool=tool,
                 )
-                log_msg = "Loaded tool id: %s, version: %s into tool panel." % (tool.id, tool.version)
+                log_msg = "Loaded tool id: {}, version: {} into tool panel.".format(tool.id, tool.version)
         else:
             inserted = False
             index = self._integrated_tool_panel.index_of_tool_id(tool_id)
@@ -382,7 +383,7 @@ class AbstractToolBox(Dictifiable, ManagesIntegratedToolPanelMixin):
                         # integrated_tool_panel.xml, so append it to the tool
                         # panel.
                         panel_dict.append_tool(tool)
-                        log_msg = "Loaded tool id: %s, version: %s into tool panel.." % (tool.id, tool.version)
+                        log_msg = "Loaded tool id: {}, version: {} into tool panel..".format(tool.id, tool.version)
                     else:
                         # We are in the process of installing the tool or we are reloading the whole toolbox.
                         tool_lineage = self._lineage_map.get(tool_id)
@@ -390,7 +391,7 @@ class AbstractToolBox(Dictifiable, ManagesIntegratedToolPanelMixin):
                         if not already_loaded:
                             # If the tool is not defined in integrated_tool_panel.xml, append it to the tool panel.
                             panel_dict.append_tool(tool)
-                            log_msg = "Loaded tool id: %s, version: %s into tool panel...." % (tool.id, tool.version)
+                            log_msg = "Loaded tool id: {}, version: {} into tool panel....".format(tool.id, tool.version)
         if log_msg and (not hasattr(self.app, 'tool_cache') or tool_id in self.app.tool_cache._new_tool_ids):
             log.debug(log_msg)
 
@@ -407,7 +408,7 @@ class AbstractToolBox(Dictifiable, ManagesIntegratedToolPanelMixin):
                 if workflow_id in self._workflows_by_id:
                     workflow = self._workflows_by_id[workflow_id]
                     self._tool_panel[key] = workflow
-                    log.debug("Loaded workflow: %s %s" % (workflow_id, workflow.name))
+                    log.debug("Loaded workflow: {} {}".format(workflow_id, workflow.name))
             elif item_type == panel_item_types.LABEL:
                 self._tool_panel[key] = val
             elif item_type == panel_item_types.SECTION:
@@ -429,7 +430,7 @@ class AbstractToolBox(Dictifiable, ManagesIntegratedToolPanelMixin):
                         if workflow_id in self._workflows_by_id:
                             workflow = self._workflows_by_id[workflow_id]
                             section.elems[section_key] = workflow
-                            log.debug("Loaded workflow: %s %s" % (workflow_id, workflow.name))
+                            log.debug("Loaded workflow: {} {}".format(workflow_id, workflow.name))
                     elif section_item_type == panel_item_types.LABEL:
                         if section_val:
                             section.elems[section_key] = section_val
@@ -636,7 +637,7 @@ class AbstractToolBox(Dictifiable, ManagesIntegratedToolPanelMixin):
             concrete_path = os.path.join(tool_path, path)
             if not os.path.exists(concrete_path):
                 # This is a lot faster than attempting to load a non-existing tool
-                raise IOError(ENOENT, os.strerror(ENOENT))
+                raise OSError(ENOENT, os.strerror(ENOENT))
             tool_shed_repository = None
             can_load_into_panel_dict = True
 
@@ -678,7 +679,7 @@ class AbstractToolBox(Dictifiable, ManagesIntegratedToolPanelMixin):
             labels = item.labels
             if labels is not None:
                 tool.labels = labels
-        except (IOError, OSError) as exc:
+        except OSError as exc:
             msg = "Error reading tool configuration file from path '%s': %s", path, unicodify(exc)
             if exc.errno == ENOENT:
                 log.error(msg)
@@ -1138,7 +1139,7 @@ def _filter_for_panel(item, item_type, filters, context):
                 if not filter_method(context, filter_item):
                     return False
             except Exception as e:
-                raise MessageException("Toolbox filter exception from '%s': %s." % (filter_method.__name__, unicodify(e)))
+                raise MessageException("Toolbox filter exception from '{}': {}.".format(filter_method.__name__, unicodify(e)))
         return True
     if item_type == panel_item_types.TOOL:
         if _apply_filter(item, filters['tool']):
@@ -1193,7 +1194,7 @@ class BaseGalaxyToolBox(AbstractToolBox):
     """
 
     def __init__(self, config_filenames, tool_root_dir, app, save_integrated_tool_panel=True):
-        super(BaseGalaxyToolBox, self).__init__(config_filenames, tool_root_dir, app, save_integrated_tool_panel)
+        super().__init__(config_filenames, tool_root_dir, app, save_integrated_tool_panel)
         old_toolbox = getattr(app, 'toolbox', None)
         if old_toolbox:
             self.dependency_manager = old_toolbox.dependency_manager
