@@ -3,7 +3,6 @@ import tempfile
 from collections import OrderedDict
 from functools import total_ordering
 
-from six import string_types, text_type
 from six.moves import shlex_quote
 
 from galaxy import exceptions
@@ -19,7 +18,7 @@ log = logging.getLogger(__name__)
 PATH_ATTRIBUTES = ["path"]
 
 
-class ToolParameterValueWrapper(object):
+class ToolParameterValueWrapper:
     """
     Base class for object that Wraps a Tool Parameter and Value.
     """
@@ -53,7 +52,7 @@ class RawObjectWrapper(ToolParameterValueWrapper):
 
     def __str__(self):
         try:
-            return "%s:%s" % (self.obj.__module__, self.obj.__class__.__name__)
+            return "{}:{}".format(self.obj.__module__, self.obj.__class__.__name__)
         except Exception:
             # Most likely None, which lacks __module__.
             return str(self.obj)
@@ -74,11 +73,11 @@ class InputValueWrapper(ToolParameterValueWrapper):
         self._other_values = other_values
 
     def _get_cast_value(self, other):
-        if self.input.type == 'boolean' and isinstance(other, string_types):
+        if self.input.type == 'boolean' and isinstance(other, str):
             return str(self)
         # For backward compatibility, allow `$wrapper != ""` for optional non-text param
         if self.input.optional and self.value is None:
-            if isinstance(other, string_types):
+            if isinstance(other, str):
                 return str(self)
             else:
                 return None
@@ -129,7 +128,7 @@ class SelectToolParameterWrapper(ToolParameterValueWrapper):
     attributes are accessible.
     """
 
-    class SelectToolParameterFieldWrapper(object):
+    class SelectToolParameterFieldWrapper:
         """
         Provide access to any field by name or index for this particular value.
         Only applicable for dynamic_options selects, which have more than simple 'options' defined (name, value, selected).
@@ -169,13 +168,13 @@ class SelectToolParameterWrapper(ToolParameterValueWrapper):
         self.fields = self.SelectToolParameterFieldWrapper(input, value, other_values, self.compute_environment)
 
     def __eq__(self, other):
-        if isinstance(other, string_types):
+        if isinstance(other, str):
             if other == '' and self.value in (None, []):
                 # Allow $wrapper == '' for select (self.value is None) and multiple select (self.value is []) params
                 return True
             return str(self) == other
         else:
-            return super(SelectToolParameterWrapper, self) == other
+            return super() == other
 
     def __ne__(self, other):
         return not self == other
@@ -186,7 +185,7 @@ class SelectToolParameterWrapper(ToolParameterValueWrapper):
         return self.input.to_param_dict_string(self.value, other_values=self._other_values)
 
     def __add__(self, x):
-        return '%s%s' % (self, x)
+        return '{}{}'.format(self, x)
 
     def __getattr__(self, key):
         return getattr(self.input, key)
@@ -203,7 +202,7 @@ class DatasetFilenameWrapper(ToolParameterValueWrapper):
     attributes are accessible.
     """
 
-    class MetadataWrapper(object):
+    class MetadataWrapper:
         """
         Wraps a Metadata Collection to return MetadataParameters wrapped
         according to the metadata spec. Methods implemented to match behavior
@@ -357,7 +356,7 @@ class DatasetFilenameWrapper(ToolParameterValueWrapper):
     __nonzero__ = __bool__
 
 
-class HasDatasets(object):
+class HasDatasets:
 
     def _dataset_wrapper(self, dataset, **kwargs):
         return DatasetFilenameWrapper(dataset, **kwargs)
@@ -406,7 +405,7 @@ class DatasetListWrapper(list, ToolParameterValueWrapper, HasDatasets):
         return dataset_instances
 
     def get_datasets_for_group(self, group):
-        group = text_type(group).lower()
+        group = str(group).lower()
         if not self._dataset_elements_cache.get(group):
             wrappers = []
             for element in self:
@@ -427,7 +426,7 @@ class DatasetListWrapper(list, ToolParameterValueWrapper, HasDatasets):
 class DatasetCollectionWrapper(ToolParameterValueWrapper, HasDatasets):
 
     def __init__(self, job_working_directory, has_collection, **kwargs):
-        super(DatasetCollectionWrapper, self).__init__()
+        super().__init__()
         self.job_working_directory = job_working_directory
         self._dataset_elements_cache = {}
         self.kwargs = kwargs
@@ -471,7 +470,7 @@ class DatasetCollectionWrapper(ToolParameterValueWrapper, HasDatasets):
         self.__element_instance_list = element_instance_list
 
     def get_datasets_for_group(self, group):
-        group = text_type(group).lower()
+        group = str(group).lower()
         if not self._dataset_elements_cache.get(group):
             wrappers = []
             for element in self.collection.dataset_elements:
@@ -525,12 +524,12 @@ class DatasetCollectionWrapper(ToolParameterValueWrapper, HasDatasets):
     __nonzero__ = __bool__
 
 
-class ElementIdentifierMapper(object):
+class ElementIdentifierMapper:
     """Track mapping of dataset collection elements datasets to element identifiers."""
 
     def __init__(self, input_datasets=None):
         if input_datasets is not None:
-            self.identifier_key_dict = dict((v, "%s|__identifier__" % k) for k, v in input_datasets.items())
+            self.identifier_key_dict = {v: "%s|__identifier__" % k for k, v in input_datasets.items()}
         else:
             self.identifier_key_dict = {}
 
