@@ -27,6 +27,9 @@ from galaxy.visualization.data_providers.cigar import get_ref_based_read_seq_and
 # Utility functions.
 #
 
+# pysam 0.16.0.1 emits logs containing the word 'Error', this can confuse the stdout/stderr checkers.
+# Can be be removed once https://github.com/pysam-developers/pysam/issues/939 is resolved.
+pysam.set_verbosity(0)
 PYSAM_INDEX_SYMLINK_NECESSARY = packaging.version.parse(pysam.__version__) <= packaging.version.parse('0.13.0')
 
 
@@ -81,7 +84,7 @@ class FeatureLocationIndexDataProvider(BaseDataProvider):
 
     def get_data(self, query):
         # Init.
-        textloc_file = open(self.converted_dataset.file_name, 'r')
+        textloc_file = open(self.converted_dataset.file_name)
         line_len = int(textloc_file.readline())
         file_len = os.path.getsize(self.converted_dataset.file_name)
         query = query.lower()
@@ -139,10 +142,10 @@ class GenomeDataProvider(BaseDataProvider):
 
     def __init__(self, converted_dataset=None, original_dataset=None, dependencies=None,
                  error_max_vals="Only the first %i %s in this region are displayed."):
-        super(GenomeDataProvider, self).__init__(converted_dataset=converted_dataset,
-                                                 original_dataset=original_dataset,
-                                                 dependencies=dependencies,
-                                                 error_max_vals=error_max_vals)
+        super().__init__(converted_dataset=converted_dataset,
+                         original_dataset=original_dataset,
+                         dependencies=dependencies,
+                         error_max_vals=error_max_vals)
 
     def write_data_to_file(self, regions, filename):
         """
@@ -270,7 +273,7 @@ class GenomeDataProvider(BaseDataProvider):
 #
 
 
-class FilterableMixin(object):
+class FilterableMixin:
     def get_filters(self):
         """ Returns a dataset's filters. """
         # Get filters.
@@ -447,7 +450,6 @@ class IntervalTabixDataProvider(TabixDataProvider, IntervalDataProvider):
     """
     Provides data from a BED file indexed via tabix.
     """
-    pass
 
 
 #
@@ -547,7 +549,6 @@ class BedTabixDataProvider(TabixDataProvider, BedDataProvider):
     """
     Provides data from a BED file indexed via tabix.
     """
-    pass
 
 
 class RawBedDataProvider(BedDataProvider):
@@ -1099,9 +1100,9 @@ class SamDataProvider(BamDataProvider):
 
     def __init__(self, converted_dataset=None, original_dataset=None, dependencies=None):
         """ Create SamDataProvider. """
-        super(SamDataProvider, self).__init__(converted_dataset=converted_dataset,
-                                              original_dataset=original_dataset,
-                                              dependencies=dependencies)
+        super().__init__(converted_dataset=converted_dataset,
+                         original_dataset=original_dataset,
+                         dependencies=dependencies)
 
         # To use BamDataProvider, original dataset must be BAM and
         # converted dataset must be BAI. Use BAI from BAM metadata.
